package com.lifethinkist.textfile;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.graphics.Typeface;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.ScrollView;
import android.widget.TextView;

import java.io.*;

import android.widget.Toast;
import com.dropbox.sync.android.*;
import com.google.android.gms.ads.AdRequest;
import com.google.android.gms.ads.AdView;
import com.google.android.gms.plus.PlusShare;

import org.apache.commons.io.FilenameUtils;

public class EditorActivity extends Activity {

    protected String sFilename = null; // Filename with extension
    final static String TAG = "TextFile";
    private boolean showDone = false;
    private boolean showShare = true;

    protected EditText editText = null;
    protected EditText filenameText = null;
    private ScrollView scrollView = null;

    private DbxAccountManager mDbxAcctMgr;
    private DbxFileSystem dbxFs;
    private static final String APP_KEY = "REMOVED";
    private static final String APP_SECRET = "REMOVED";

    private AdView adViewEditor;

    //---- Create and Start

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        //---- Setup

        // Setup
        super.onCreate(savedInstanceState);

        // Setup views
        setContentView(R.layout.layout_editor);

        // Setup admob ads
        adViewEditor = (AdView) this.findViewById(R.id.adViewEditor);
        AdRequest adRequest = new AdRequest.Builder()
                .addTestDevice(AdRequest.DEVICE_ID_EMULATOR) // Emulator
                .addTestDevice("489AF3AD9E98ECCE3B7979B067D95318") // Lenovo Tablet
                .build();
        adViewEditor.loadAd(adRequest);

        // Setup action bar
        getActionBar().setDisplayShowHomeEnabled(true);
        getActionBar().setDisplayShowTitleEnabled(true);
        getActionBar().setHomeButtonEnabled(true);
        getActionBar().setDisplayHomeAsUpEnabled(true);

        // Get controls
        filenameText = (EditText)findViewById(R.id.actionfile);
        editText = (EditText)findViewById(R.id.note);
        scrollView = (ScrollView)findViewById(R.id.scroll2);

        // Setup typeface
        Typeface typeface = Typeface.createFromAsset(getAssets(),"fonts/helvetica.otf");
        editText.setTypeface(typeface);
        editText.setTextSize(TypedValue.COMPLEX_UNIT_SP, 15);

        // Setup scrollview
        scrollView.isSmoothScrollingEnabled();

        // Remove mouse cursor from text window
        findViewById(R.id.layout_editor).requestFocus();

        // Setup dropbox
        mDbxAcctMgr = DbxAccountManager.getInstance(getApplicationContext(), APP_KEY, APP_SECRET);
        if (mDbxAcctMgr.hasLinkedAccount()){
            try {
                dbxFs = DbxFileSystem.forAccount(mDbxAcctMgr.getLinkedAccount());
            } catch (IOException e) {
                Toast.makeText(getApplicationContext(), "Connection to Dropbox failed", Toast.LENGTH_LONG).show();
            }
        }

        //---- Listeners

        // Edit file when click or focus on screen
        editText.setOnFocusChangeListener(new View.OnFocusChangeListener(){
            public void onFocusChange(View v, boolean hasFocus){
                if(hasFocus){
                    showDone = true;
                    showShare = false;
                    showKeyboard();
                    invalidateOptionsMenu();
                }
                if(!hasFocus){
                    showDone = false;
                    showShare = true;
                    hideKeyboard();
                    invalidateOptionsMenu();
                    saveFile(sFilename);
                }
            }
        });

        // Rename when filename changed
        filenameText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            public void onFocusChange(View v, boolean hasFocus) {
                if(hasFocus==true){
                    showDone = true;
                    showShare = false;
                    invalidateOptionsMenu();
                }
                if(!hasFocus){
                    filenameText.setSelection(0,0);
                    showDone = false;
                    showShare = true;
                    invalidateOptionsMenu();
                    renameFile();
                }
            }
        });

        //---- Get Action

        // Get bundle
        String action = null;
        String filename = null;
        Bundle extras = getIntent().getExtras();

        // Get action and filename
        if (extras != null) {
            action = extras.getString("action");
            filename = extras.getString("filename");
        }

        // Choose action to take
        if (action != null) {
            if (action.equals("open_file")) {
                openFile(filename);
            } else if (action.equals("new_file")) {
                newFile();
            }
        }

        restoreState();
    }

    @Override
    protected void onStart(){
        super.onStart();
    }

    //---- Pause and Resume

    @Override
    protected void onPause(){
        // Pause the AdView.
        adViewEditor.pause();

        super.onPause();

        // Rename and save note
        hideKeyboard();
        renameFile();
        saveFile(sFilename);
    }

    @Override
    protected void onResume(){
        super.onResume();

        // Resume the AdView.
        adViewEditor.resume();
    }

    @Override
    protected void onStop(){
        super.onStop();
    }

    public void onDestroy() {
        // Destroy the AdView.
        adViewEditor.destroy();

        super.onDestroy();
    }

    //---- Start and Stop

    public void restoreState(){
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        Boolean toggleShowLocalFiles = sharedPreferences.getBoolean("settings_show_local_files", true);
        Boolean togglePlusFeatures = sharedPreferences.getBoolean("settings_enable_plus_features", false);
        Boolean toggleProFeatures = sharedPreferences.getBoolean("settings_enable_pro_features", false);
        Boolean togglePremiumFeatures = sharedPreferences.getBoolean("settings_enable_premium_features", false);

        // Restore ads
        if (togglePlusFeatures || toggleProFeatures || togglePremiumFeatures){
            hideAds();
        } else {
            showAds();
        }
    }

    //---- Menus

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {

        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_editor, menu);
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu){

        // Show or hide done button on action bar
        MenuItem item = menu.findItem(R.id.done);
        MenuItem item4 = menu.findItem(R.id.action_share);
        item.setVisible(showDone);
        item4.setVisible(showShare);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                hideKeyboard();

                // app icon in action bar clicked; go home
                launchFilesActivity();
                return true;
            case R.id.done:
                hideKeyboard();
                showDone = false;
                showShare = true;
                invalidateOptionsMenu();
                findViewById(R.id.layout_editor).requestFocus();
                return true;
            case R.id.action_share:
                displayShareDialog();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    //---- Activities

    // Launch main activity - rename and save done onPause
    public void launchFilesActivity (){
        Intent intent = new Intent(this, FilesActivity.class);
        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        this.startActivity(intent);
        this.overridePendingTransition(0,0);
    }

    public void launchWebsite(){
        Intent browserIntent = new Intent(Intent.ACTION_VIEW, Uri.parse("http://www.textfileapp.com"));
        startActivity(browserIntent);
    }

    //----  Dialogs

    // Display share dialog
    public void displayShareDialog() {
        LayoutInflater factory = LayoutInflater.from(this);
        final View shareDialogView = factory.inflate(R.layout.dialog_share_textfile, null);
        final AlertDialog shareDialog = new AlertDialog.Builder(this).create();
        shareDialog.setView(shareDialogView);

        shareDialogView.findViewById(R.id.dialog_rate_textfile_play_store).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                rateTextFile();
                shareDialog.dismiss();
            }
        });

        shareDialogView.findViewById(R.id.dialog_share_textfile_google).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                shareTextFileGoogle();
                shareDialog.dismiss();
            }
        });

        shareDialogView.findViewById(R.id.dialog_share_textfile_facebook).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                shareTextFileFacebook();
                shareDialog.dismiss();
            }
        });

        shareDialogView.findViewById(R.id.dialog_share_textfile_twitter).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                shareTextFileTwitter();
                shareDialog.dismiss();
            }
        });

        shareDialogView.findViewById(R.id.dialog_share_textfile_email).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                shareTextFileEmail();
                shareDialog.dismiss();
            }
        });

        // Hide more for now
/*
        shareDialogView.findViewById(R.id.dialog_share_textfile_more).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                shareTextFileMore();
                shareDialog.dismiss();
            }
        });
*/

        shareDialogView.findViewById(R.id.dialog_share_textfile_cancel).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                shareDialog.dismiss();
            }
        });

        shareDialog.show();
    }

    //---- Dialog Actions

    public void rateTextFile(){

        boolean isPlayStoreInstalled;

        isPlayStoreInstalled = isPackageInstalled("com.android.vending", this);

        if (!isPlayStoreInstalled){
            Toast.makeText(getApplicationContext(), "Play Store not installed. Install Google Play Store to rate TextFile.", Toast.LENGTH_LONG).show();
        } else {
            String appPackageName = getPackageName();
            Intent marketIntent = new Intent(Intent.ACTION_VIEW, Uri.parse("market://details?id=" + appPackageName));
            marketIntent.addFlags(Intent.FLAG_ACTIVITY_NO_HISTORY | Intent.FLAG_ACTIVITY_CLEAR_WHEN_TASK_RESET | Intent.FLAG_ACTIVITY_MULTIPLE_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
            startActivity(marketIntent);
        }
    }

    public void shareTextFileGoogle(){

        boolean isGooglePlayServicesInstalled;
        isGooglePlayServicesInstalled = isPackageInstalled("com.google.android.gms", this);

        if (!isGooglePlayServicesInstalled) {
            // Option 1 - Browser
            String url = "https://plus.google.com/share?url=http://www.textfileapp.com";
            Intent i = new Intent(Intent.ACTION_VIEW);
            i.setData(Uri.parse(url));
            startActivity(i);
        } else {
            // Option 2 - Google+ share intent
            Intent shareIntent = new PlusShare.Builder(this)
                    .setType("text/plain")
                    .setText(this.getString(R.string.text_textfile_share_google_text))
                    .setContentUrl(Uri.parse("http://www.textfileapp.com"))
                    .getIntent();

            startActivityForResult(shareIntent, 0);
        }
    }

    public void shareTextFileFacebook(){
        String url = "https://m.facebook.com/sharer.php?u=http://www.textfileapp.com";
        Intent i = new Intent(Intent.ACTION_VIEW);
        i.setData(Uri.parse(url));
        startActivity(i);
    }

    public void shareTextFileTwitter(){
        String url = "http://www.twitter.com/intent/tweet?text=" + this.getString(R.string.text_textfile_share_twitter_text);
        Intent i = new Intent(Intent.ACTION_VIEW);
        i.setData(Uri.parse(url));
        startActivity(i);
    }

    public void shareTextFileEmail(){
        Intent emailIntent = new Intent(android.content.Intent.ACTION_SEND);

        emailIntent.setType("plain/text");
        emailIntent.putExtra(android.content.Intent.EXTRA_EMAIL, new String[]{""});
        emailIntent.putExtra(android.content.Intent.EXTRA_SUBJECT, this.getString(R.string.text_textfile_share_email_title));
        emailIntent.putExtra(android.content.Intent.EXTRA_TEXT, this.getString(R.string.text_textfile_share_email_text));

        startActivity(emailIntent);
    }

    public void shareTextFileMore(){
        Intent share = new Intent(android.content.Intent.ACTION_SEND);

        share.setType("text/plain");
        share.addFlags(Intent.FLAG_ACTIVITY_CLEAR_WHEN_TASK_RESET);
        share.putExtra(Intent.EXTRA_SUBJECT, this.getString(R.string.text_textfile_share_default_subject));
        share.putExtra(Intent.EXTRA_TEXT, "http://www.textfileapp.com");

        startActivity(Intent.createChooser(share, this.getString(R.string.text_textfile_share_default_dialog_title)));
    }

    //---- Choose File Service

    public void openFile(String filename){
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        Boolean toggleShowLocalFiles = sharedPreferences.getBoolean("settings_show_local_files", true);

        if (toggleShowLocalFiles == true) {
            sFilename = filename + ".txt";
            openFileLocal(sFilename);
//            openFileLocal(filename);
        } else {
            openFileDropbox(filename);
        }
    }

    public void saveFile(String filename){
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        Boolean toggleShowLocalFiles = sharedPreferences.getBoolean("settings_show_local_files", true);

        if (toggleShowLocalFiles == true) {
            saveFileLocal(filename);
        } else {
            saveFileDropbox(filename);
        }
    }

    public void renameFile(){
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        Boolean toggleShowLocalFiles = sharedPreferences.getBoolean("settings_show_local_files", true);

        if (toggleShowLocalFiles == true) {
            renameFileLocal();
        } else {
            renameFileDropbox();
        }
    }

    public void newFile(){
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        Boolean toggleShowLocalFiles = sharedPreferences.getBoolean("settings_show_local_files", true);

        if (toggleShowLocalFiles == true) {
            newFileLocal();
        } else {
            newFileDropbox();
        }
    }

    //---- Local File Actions

    public void openFileLocal(String filename){
        String text = "";

        // Open and read the file
        try {
            FileInputStream fis = openFileInput(filename);
            InputStreamReader isr = new InputStreamReader(fis);
            BufferedReader br = new BufferedReader(isr);
            String s;
            text = br.readLine();
            while((s=br.readLine())!=null){
                text = text + "\n" + s;
            }
            br.close();
            isr.close();
            fis.close();
        } catch (IOException e){
            e.printStackTrace();
        }

        // Display interface and set filename
        editText.setText(text);
        displayFilenameNoExtension(filename);
        sFilename=filename;
    }

    // - Filename argument must contain file extension
    public void saveFileLocal(String filename){
        String content = editText.getText().toString();

        // Create a file
        try {
            FileOutputStream file = openFileOutput(filename, Context.MODE_PRIVATE);
            file.write(content.getBytes());
            file.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void renameFileLocal(){
        String fileName = filenameText.getText().toString()+".txt";
        File directory = getFilesDir();
        File fOld = new File(directory, sFilename);
        File fNew = new File(directory,  fileName);
        fOld.renameTo(fNew);

        // Reset filename
        sFilename=fileName;
    }

    public void newFileLocal() {
        String text = "";
        String filename = createUniqueLocalFilename();

        // Create a file
        try {
            FileOutputStream file = openFileOutput(filename, Context.MODE_PRIVATE);
            file.write(text.getBytes());
            file.flush();
            file.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        sFilename = filename;

        // Display filename in title bar
        displayFilenameNoExtension(sFilename);

        // Select filename in title bar
        filenameText.requestFocus();

        // Show keyboard
        InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
        imm.toggleSoftInput(InputMethodManager.SHOW_FORCED,0);
    }

    //---- Dropbox File Actions

    public void openFileAtRootDropbox(String filename){
        String text = "";

        try {
            DbxPath path = new DbxPath("/"+filename);
            DbxFile file = dbxFs.open(path);
            text = file.readString();
            file.close();
        } catch (IOException e) {
            hideKeyboard();
            launchFilesActivity();
            Toast.makeText(getApplicationContext(), getString(R.string.error_open_dropbox_file), Toast.LENGTH_LONG).show();
        }

        // Display interface and set filename
        editText.setText(text);
        displayFilenameNoExtension(filename);
        sFilename=filename;
    }

//    public void openFileDropbox(String filepath) {
//        String text = "";
//        String filename = "";
//
//        DbxPath path = new DbxPath(filepath);
//        DbxFile file = null;
//        try {
//            file = dbxFs.open(path);
//            filename = path.getName();
//            text = file.readString();
//
//            // Display interface and set filename
//            editText.setText(text);
//            displayFilenameNoExtension(filename);
//            sFilename=filename;
//        } catch (IOException e) {
//            hideKeyboard();
//            launchFilesActivity();
//            Toast.makeText(getApplicationContext(), "Opening of Dropbox file failed. To edit a dropbox file when offline, open the file at least once when online.", Toast.LENGTH_LONG).show();
//        } finally {
//            file.close();
//        }
//    }


    public void openFileDropbox(String filepath) {
        String text = "";
        String filename = "";

        DbxPath path = new DbxPath(filepath);
        DbxFile dbxFile = null;

        try {

            // Set file and filename
            dbxFile = dbxFs.open(path);
            filename = path.getName();

            DbxFileStatus status = dbxFile.getSyncStatus();

            // If file not cached
            if(!status.isCached){
//                Toast.makeText(getApplicationContext(), "File not cached", Toast.LENGTH_LONG).show();
                try {
                    text = dbxFile.readString();

                    // Display interface and set filename
                    editText.setText(text);
                    displayFilenameNoExtension(filename);
                    sFilename=filename;

                } catch (IOException e) {
                    hideKeyboard();
                    launchFilesActivity();
                    Toast.makeText(getApplicationContext(), "Opening of Dropbox file failed. To edit a dropbox file when offline, open the file at least once when online.", Toast.LENGTH_LONG).show();
                }

            // If file is cached
            } else {
//                Toast.makeText(getApplicationContext(), "File cached", Toast.LENGTH_LONG).show();

                // If latest version
                if(status.isLatest){
//                    Toast.makeText(getApplicationContext(), "File is latest", Toast.LENGTH_LONG).show();
                    try {
                        text = dbxFile.readString();

                        // Display interface and set filename
                        editText.setText(text);
                        displayFilenameNoExtension(filename);
                        sFilename=filename;

                    } catch (IOException e) {
                        hideKeyboard();
                        launchFilesActivity();
                        Toast.makeText(getApplicationContext(), "Opening of Dropbox file failed. To edit a dropbox file when offline, open the file at least once when online.", Toast.LENGTH_LONG).show();
                    }


                // If not latest version
                } else {
//                    Toast.makeText(getApplicationContext(), "File not latest", Toast.LENGTH_LONG).show();
                    DbxFileStatus updated = dbxFile.getNewerStatus();

                    // Check if cached
                    while (!updated.isCached){
                        updated = dbxFile.getNewerStatus();
                    }

                    // Update and read the file
                    dbxFile.update();

                    // Check if cached
                    while (!updated.isCached){
                        updated = dbxFile.getNewerStatus();
                    }

                    try {
                        text = dbxFile.readString();

                        // Display interface and set filename
                        editText.setText(text);
                        displayFilenameNoExtension(filename);
                        sFilename=filename;

                    } catch (IOException e) {
                        hideKeyboard();
                        launchFilesActivity();
                        Toast.makeText(getApplicationContext(), "Opening of Dropbox file failed. To edit a dropbox file when offline, open the file at least once when online.", Toast.LENGTH_LONG).show();
                    }
                }
            }
        } catch (IOException e) {
            hideKeyboard();
            launchFilesActivity();
            Toast.makeText(getApplicationContext(), "Opening of Dropbox file failed. To edit a dropbox file when offline, open the file at least once when online.", Toast.LENGTH_LONG).show();
        } finally {
            dbxFile.close();
        }
    }

    // Todo: Check if file exists somehow?
    public void saveFileDropbox(String filename){

        // Get path from preferences
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        String currentDbxPath = sharedPreferences.getString("setting_current_dbx_path", DbxPath.ROOT.toString());
        DbxPath dbxPath = new DbxPath(currentDbxPath);
        DbxPath dbxFile = dbxPath.getChild(filename);

        String content = editText.getText().toString();
        DbxFile file = null;

        try {
            file = dbxFs.open(dbxFile);
            file.writeString(content);
            file.close();
        } catch (DbxException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void renameFileDropbox(){

        // Get path from preferences
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        String currentDbxPath = sharedPreferences.getString("setting_current_dbx_path", DbxPath.ROOT.toString());
        DbxPath dbxPath = new DbxPath(currentDbxPath);

        String fileName = filenameText.getText().toString()+".txt";
        DbxPath dbxPathOld = dbxPath.getChild(sFilename);
        DbxPath dbxPathNew = dbxPath.getChild(fileName);

        try {
            dbxFs.move(dbxPathOld, dbxPathNew);
        } catch (DbxException e) {
            e.printStackTrace();
        }
        sFilename=fileName;
    }

    public void newFileDropbox() {

        // Get path from preferences
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        String currentDbxPath = sharedPreferences.getString("setting_current_dbx_path", DbxPath.ROOT.toString());
        DbxPath dbxPath = new DbxPath(currentDbxPath);

        String text = "";
        String filename = createUniqueDropboxFilename();
        DbxFile testFile = null;

        try {
            testFile = dbxFs.create(dbxPath.getChild(filename));
        } catch (IOException e) {
            e.printStackTrace();
        }
        try {
            testFile.writeString(text);
            testFile.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        sFilename = filename;

        // Display filename in title bar
        displayFilenameNoExtension(sFilename);

        // Select filename in title bar
        filenameText.requestFocus();

        // Show keyboard
        InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
        imm.toggleSoftInput(InputMethodManager.SHOW_FORCED,0);
    }

    //---- Settings

    public void showAds(){
//        LinearLayout layout = (LinearLayout)findViewById(R.id.layout_files);
//        AdView adViewFiles = (AdView) layout.findViewById(R.id.adViewFiles);

        AdView adViewEditor = (AdView) this.findViewById(R.id.adViewEditor);
        adViewEditor.setVisibility(View.VISIBLE);
    }

    public void hideAds(){
//        LinearLayout layout = (LinearLayout)findViewById(R.id.layout_files);
//        AdView adViewFiles = (AdView) layout.findViewById(R.id.adViewFiles);

        AdView adViewEditor = (AdView) this.findViewById(R.id.adViewEditor);
        adViewEditor.setVisibility(View.GONE);
    }

    //---- Utilities

    public String createUniqueLocalFilename() {
        String filename = "Untitled-1.txt";
        int i=1;
        File file = getBaseContext().getFileStreamPath(filename);

        while (file.exists()){
            i++;
            filename = "Untitled-"+i+".txt";
            file = getBaseContext().getFileStreamPath(filename);
        }
        return filename;
    }

    public String createUniqueDropboxFilename() {

        // Get path from preferences
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        String currentDbxPath = sharedPreferences.getString("setting_current_dbx_path", DbxPath.ROOT.toString());
        DbxPath dbxPath = new DbxPath(currentDbxPath);

        String filename = "Untitled-1.txt";
        int i=1;
        DbxPath path = dbxPath.getChild(filename);

        try {
            while (dbxFs.isFile(path)){
                i++;
                filename = "Untitled-"+i+".txt";
                path = dbxPath.getChild(filename);
            }
        } catch (DbxException e) {
            e.printStackTrace();
        }
        return filename;
    }

    public void displayFilenameNoExtension(String s){
        String sFileNoExtension = FilenameUtils.removeExtension(s);
        filenameText.setText(sFileNoExtension, TextView.BufferType.EDITABLE);
    }

    public void hideKeyboard(){
        InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
        imm.hideSoftInputFromWindow(editText.getWindowToken(), 0);
    }

    public void showKeyboard(){
        InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
        imm.toggleSoftInput(InputMethodManager.SHOW_FORCED,0);
    }

    // Check if network connection is online
    private boolean isNetworkAvailable() {
        ConnectivityManager connectivityManager
                = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetworkInfo = connectivityManager.getActiveNetworkInfo();
        return activeNetworkInfo != null && activeNetworkInfo.isConnected();
    }

    // Check if app is installed
    private boolean isPackageInstalled(String packagename, Context context) {
        PackageManager pm = context.getPackageManager();
        try {
            pm.getPackageInfo(packagename, PackageManager.GET_ACTIVITIES);
            return true;
        } catch (PackageManager.NameNotFoundException e) {
            return false;
        }
    }
}