package com.lifethinkist.textfile;

import android.accounts.Account;
import android.accounts.AccountManager;
import android.app.AlertDialog;
import android.app.ListActivity;
import android.content.ClipData;
import android.content.ClipboardManager;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.text.SpannableString;
import android.text.method.LinkMovementMethod;
import android.text.util.Linkify;
import android.view.*;
import android.widget.*;

import java.io.*;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.android.vending.billing.util.IabHelper;
import com.android.vending.billing.util.IabResult;
import com.android.vending.billing.util.Inventory;
import com.dropbox.sync.android.*;
import com.google.android.gms.ads.AdRequest;
import com.google.android.gms.ads.AdView;

import org.apache.commons.io.FilenameUtils;

public class FilesActivity extends ListActivity {

    //private boolean showNewFolderButton = true;
    private boolean showSyncButton = false;

    protected ArrayList<String>files = new ArrayList<String>();
    protected List<String>filesDropbox = new ArrayList<String>();

    protected ArrayList<DbxFileInfo>filesNewDropbox = new ArrayList<DbxFileInfo>();
    protected ArrayList<DbxFileInfo>foldersNewDropbox = new ArrayList<DbxFileInfo>();
    protected ArrayList<DbxFileInfo>filesAndFoldersDropbox = new ArrayList<DbxFileInfo>();

    private DbxAccountManager mDbxAcctMgr;
    private DbxFileSystem dbxFs;
    private DbxPath dbxPath = DbxPath.ROOT;

    private static final String APP_KEY = "REMOVED";
    private static final String APP_SECRET = "REMOVED";

    IabHelper mHelper;
    private String checkedString;

    // Upgrade SKU IDs
    // Pro (source code) = Premium (upgrade)
    // Premium (source code) = Professional (upgrade)
//    static final String SKU_PRO = "subscription.pro.month";
//    static final String SKU_PREMIUM = "subscription.premium.month";
    static final String SKU_PRO = "upgrade.pro";
    static final String SKU_PREMIUM = "upgrade.premium";
    protected Button proFeaturesButton;
    protected Button premiumFeaturesButton;
    boolean hasPro;
    boolean hasPremium;

    URL shareURL;

    final static String TAG = "TextFile";

    private LinearLayout textInfoBar;
    private LinearLayout newsInfoBar;
    private TextView textInfoBarText;
    private TextView textNewsBarText;

    private AdView adViewFiles;

    //---- Create and Start

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        // Setup
        super.onCreate(savedInstanceState);
        String base64EncodedPublicKey;

        // Setup views
        setContentView(R.layout.layout_files);
        registerForContextMenu(getListView());

        // Setup hidden info bar
        textInfoBar = (LinearLayout)findViewById(R.id.text_info_bar);
        textInfoBarText = (TextView)findViewById(R.id.text_info_bar_text);

        // Setup action bar - ShowFiles methods change action bar too
        getActionBar().setTitle(R.string.files_title);
        getActionBar().setDisplayShowHomeEnabled(false);
        getActionBar().setDisplayShowTitleEnabled(true);
        getActionBar().setHomeButtonEnabled(false);
        getActionBar().setDisplayHomeAsUpEnabled(false);

        // Setup admob ads
        adViewFiles = (AdView) this.findViewById(R.id.adViewFiles);
        AdRequest adRequest = new AdRequest.Builder()
                .addTestDevice(AdRequest.DEVICE_ID_EMULATOR) // Emulator
                .addTestDevice("489AF3AD9E98ECCE3B7979B067D95318") // Lenovo Tablet
                .build();
        adViewFiles.loadAd(adRequest);

        // Make textfileapp.com file only on first run
        SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
        if(!prefs.getBoolean("firstTime", false)) {

            // One time code
            createWebsiteFile();

            SharedPreferences.Editor editor = prefs.edit();
            editor.putBoolean("firstTime", true);
            editor.commit();
        }

        // Setup dropbox
        mDbxAcctMgr = DbxAccountManager.getInstance(getApplicationContext(), APP_KEY, APP_SECRET);
        if (mDbxAcctMgr.hasLinkedAccount()){
            try {
                dbxFs = DbxFileSystem.forAccount(mDbxAcctMgr.getLinkedAccount());
            } catch (IOException e) {
                Toast.makeText(getApplicationContext(), "Connection to Dropbox failed", Toast.LENGTH_LONG).show();
            }
        }

        // Setup billing
        base64EncodedPublicKey = "REMOVED";
        checkedString = checkString(base64EncodedPublicKey);
        mHelper = new IabHelper(this, checkedString);

        proFeaturesButton = (Button)findViewById(R.id.proFeaturesButton);
        premiumFeaturesButton = (Button)findViewById(R.id.premiumFeaturesButton);

        // Check for account and Play Store and set purchases
        checkGoogleAccountPlayStoreSetPurchases();

        // Restore state depending on purchases and local or dropbox files
//        restoreState();
    }

    @Override
    protected void onStart(){
        super.onStart();
    }

    //---- Pause and Resume

    @Override
    protected void onPause(){
        // Pause the AdView.
        adViewFiles.pause();

        super.onPause();

        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        Boolean toggleShowLocalFiles = sharedPreferences.getBoolean("settings_show_local_files", true);

        if (toggleShowLocalFiles == true) {
            // Do nothing
        } else {
            // Remove folder listener
            removeFolderListener();
        }
    }

    @Override
    protected void onResume(){
        super.onResume();

        // Resume the AdView.
        adViewFiles.resume();

        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        Boolean toggleShowLocalFiles = sharedPreferences.getBoolean("settings_show_local_files", true);

        if (toggleShowLocalFiles == true) {
            // Do nothing
        } else {
            // Add folder listener
            addFolderListener();
        }
        restoreState();
    }

    @Override
    protected void onStop(){
        super.onStop();
    }

    //---- Start and Stop

    // View files
    public void restoreState(){
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        Boolean toggleShowLocalFiles = sharedPreferences.getBoolean("settings_show_local_files", true);
        Boolean toggleProFeatures = sharedPreferences.getBoolean("settings_enable_pro_features", false);
        Boolean togglePremiumFeatures = sharedPreferences.getBoolean("settings_enable_premium_features", false);
        Boolean toggleSaleOn = sharedPreferences.getBoolean("settings_enable_sale_on", false);
        int saleSavePercent = sharedPreferences.getInt("sale_save_percent", 0);
        Boolean settingsNewsBarClicked = sharedPreferences.getBoolean("settings_news_clicked", false);

        // Restore file list
        if (toggleShowLocalFiles) {
            showFilesLocal();
/*
            showNewFolderButton = false;
            showSyncButton = false;
            invalidateOptionsMenu();
*/
        } else if (toggleProFeatures || togglePremiumFeatures){
            showFilesFoldersDropbox();
//            addFolderListener();

            textInfoBar.setVisibility(View.GONE);
/*
            showNewFolderButton = true;
            showSyncButton = true;
            invalidateOptionsMenu();
*/
        } else {
            showFilesDropbox();
//            addFolderListener();
/*
            showNewFolderButton = true;
            showSyncButton = true;
            invalidateOptionsMenu();
*/
        }

        // Restore ads
        if (toggleProFeatures || togglePremiumFeatures){
            hideAds();
        } else {
            showAds();
        }

        // Set info bar text if sale on
        if (toggleSaleOn && !toggleProFeatures && !togglePremiumFeatures){
            textInfoBarText.setText("Save " + saleSavePercent + "%. Upgrade today.\n• Tap here for more information.");
            textInfoBarText.setTextColor(getResources().getColor(R.color.red));
            showInfoBar();
        } else {
            textInfoBarText.setText(R.string.file_limit);
            textInfoBarText.setTextColor(getResources().getColor(R.color.holo_light_ui_text));
        }

        // Show or hide news bar
        if (!settingsNewsBarClicked){
            newsInfoBar = (LinearLayout)findViewById(R.id.text_news_bar);
            newsInfoBar.setVisibility(View.VISIBLE);
        }

        // Remove news preference
//        SharedPreferences.Editor editor = sharedPreferences.edit();
//        editor.remove("settings_news_clicked");
//        editor.commit();
    }

    //---- Menus

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {

        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_files, menu);
        return true;
    }

    // Menu
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {

            // Visible
            case android.R.id.home:
                parentFolderDropbox();
                return true;
            case R.id.action_parent_folder:
                parentFolderDropbox();
                return true;
            case R.id.action_new_file:
                newFile();
                return true;
            case R.id.action_new_folder:
                newFolder();
                return true;
            case R.id.action_pro:
                launchPurchase();
                return true;
            case R.id.action_settings:
                launchSettings();
                return true;
            case R.id.action_about:
                displayAboutDialog();
                return true;

            // Hidden
            case R.id.action_sync:
//                syncFilesDropbox();
                return true;
            case R.id.action_remove_ads:
                displayRemoveAdsDialog();
                return true;
            case R.id.action_share_link:
                displayDisabledShareLinkDialog();
                return true;

            // Features
            case R.id.action_enable_plus:
//                enableProFeatures();
                disableAllFeatures();
                return true;
            case R.id.action_enable_pro:
                enableProFeatures();
                return true;
            case R.id.action_enable_premium:
                enablePremiumFeatures();
                return true;
           default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu){

        // Show or hide new folder button on action bar
        /*MenuItem item = menu.findItem(R.id.action_new_folder);
        item.setVisible(showNewFolderButton);*/
        MenuItem item2 = menu.findItem(R.id.action_sync);
        item2.setVisible(showSyncButton);

        return true;
    }

    @Override
    public void onCreateContextMenu(ContextMenu menu, View v, ContextMenu.ContextMenuInfo menuInfo) {
        super.onCreateContextMenu(menu, v, menuInfo);
        MenuInflater inflater = getMenuInflater();

        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        Boolean toggleShowLocalFiles = sharedPreferences.getBoolean("settings_show_local_files", true);

        // Different context menu if local or dropbox
        if (toggleShowLocalFiles == true) {
            inflater.inflate(R.menu.context_files_file, menu);
        } else {
            // Get the list item using adapter
            AdapterView.AdapterContextMenuInfo info;
            info = (AdapterView.AdapterContextMenuInfo) menuInfo;
            DbxFileInfo dbxFileInfo = (DbxFileInfo) getListAdapter().getItem(info.position);

            // Decide which context menu to display
            if (dbxFileInfo.isFolder) {
                inflater.inflate(R.menu.context_files_folder, menu);
            } else {
                inflater.inflate(R.menu.context_files_file, menu);
            }
        }
    }

    // Context menu
    @Override
    public boolean onContextItemSelected(MenuItem item) {
        AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo) item.getMenuInfo();
        switch (item.getItemId()) {
            case R.id.action_delete_context:
                displayDeleteDialog(info.position);
                return true;
            case R.id.action_rename_context:
                displayRenameFolderDialog(info.position);
                return true;
            case R.id.action_share_context:
                shareFileOrFolder(info.position);
                return true;
            default:
                return super.onContextItemSelected(item);
        }
    }

    //---- Info Bar

    public void showHideInfoBar(){

    }

    //---- Dialogs

    // Delete file dialog
    public void displayDeleteDialog(int position) {
        final int tempPosition = position;
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setMessage(R.string.delete_heading);
        builder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                deleteFileOrFolder(tempPosition);
            }
        });
        builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.cancel();
            }
        });
        AlertDialog deleteFileDialog = builder.create();
        deleteFileDialog.show();
    }

    // Display dialog for creating new folder locally
    public void displayLocalNewFolderDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setMessage(R.string.dialog_new_folder_local);
        builder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                launchSettings();
            }
        });
        builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.cancel();
            }
        });
        AlertDialog deleteFileDialog = builder.create();
        deleteFileDialog.show();
    }

    // Display dialog for creating new folder on dropbox with free version
    public void displayDialogNewFolderDropboxFree() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setMessage(R.string.dialog_new_folder_dropbox_free);
        builder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                launchPurchase();
            }
        });
        builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.cancel();
            }
        });
        AlertDialog deleteFileDialog = builder.create();
        deleteFileDialog.show();
    }

    // Display rename folder dialog
    public void displayRenameFolderDialog(int position) {
        final int tempPosition = position;

        LayoutInflater factory = LayoutInflater.from(this);
        final View renameDialogView = factory.inflate(R.layout.dialog_rename_folder, null);
        final AlertDialog deleteDialog = new AlertDialog.Builder(this).create();
        deleteDialog.setView(renameDialogView);

        DbxFileInfo dbxFileInfo = (DbxFileInfo) getListAdapter().getItem(position);
        final EditText editText = (EditText)renameDialogView.findViewById(R.id.rename_folder_edittext);
        editText.setText(dbxFileInfo.path.getName());

        renameDialogView.findViewById(R.id.rename_folder_ok).setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                // Send list item position and entered text to rename folder
                String folderName = editText.getText().toString();
                renameFolderDropbox(tempPosition, folderName);

                deleteDialog.dismiss();
            }
        });

        renameDialogView.findViewById(R.id.rename_folder_cancel).setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                deleteDialog.dismiss();

            }
        });

        deleteDialog.show();
    }

    // Display new folder dialog
    public void displayDropboxNewFolderDialog() {
        LayoutInflater factory = LayoutInflater.from(this);
        final View deleteDialogView = factory.inflate(R.layout.dialog_new_folder, null);
        final AlertDialog deleteDialog = new AlertDialog.Builder(this).create();
        deleteDialog.setView(deleteDialogView);

        final EditText editText = (EditText)deleteDialogView.findViewById(R.id.new_folder_edittext);

        deleteDialogView.findViewById(R.id.new_folder_ok).setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                //your business logic
                String folderName = editText.getText().toString();
                newFolderDropbox(folderName);

                deleteDialog.dismiss();
            }
        });

        deleteDialogView.findViewById(R.id.new_folder_cancel).setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                deleteDialog.dismiss();

            }
        });
        deleteDialog.show();
    }

    // Display dialog for removing ads in main menu
    public void displayRemoveAdsDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setMessage(R.string.dialog_remove_ads);
        builder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                launchPurchase();
            }
        });
        builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.cancel();
            }
        });
        AlertDialog deleteFileDialog = builder.create();
        deleteFileDialog.show();
    }

    // Display dialog for local for selecting share dialog in main menu
    public void displayLocalMenuShareDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setMessage(R.string.dialog_share_link_disabled);
        builder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                launchPurchase();
            }
        });
        builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.cancel();
            }
        });
        AlertDialog deleteFileDialog = builder.create();
        deleteFileDialog.show();
    }

    // Display enabled share link dialog
    public void displayEnabledShareLinkDialog(int position) {
        final int tempPosition = position;
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setMessage(R.string.share_link_heading);
        builder.setPositiveButton(R.string.copy, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                shareLinkCopy(tempPosition);
            }
        });
        builder.setNeutralButton(R.string.email, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                shareLinkEmail(tempPosition);
            }
        });
        builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.cancel();
            }
        });
        AlertDialog deleteFileDialog = builder.create();
        deleteFileDialog.show();
    }

    // Display disabled share link dialog
    public void displayDisabledShareLinkDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setMessage(R.string.dialog_share_link_disabled);
        builder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                launchSettings();
            }
        });
        builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.cancel();
            }
        });
        AlertDialog deleteFileDialog = builder.create();
        deleteFileDialog.show();
    }

    // Display about dialog with TextFile info
    public void displayAboutDialog() {

        // Linkify the message
        final SpannableString m = new SpannableString(this.getText(R.string.dialog_files_about));
        Linkify.addLinks(m, Linkify.WEB_URLS);

        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setMessage(R.string.dialog_files_about);
//        builder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
//            public void onClick(DialogInterface dialog, int id) {
//                launchPurchase();
//            }
//        });
        builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.cancel();
            }
        });
        AlertDialog deleteFileDialog = builder.create();
        deleteFileDialog.show();

        // Make the TextView containing message clickable
        ((TextView)deleteFileDialog.findViewById(android.R.id.message)).setMovementMethod(LinkMovementMethod.getInstance());
    }

    // Display no Play Store error dialog
    public void displayNoPlayStoreDialog(){
        AlertDialog.Builder alertBuilder = new AlertDialog.Builder(this);
        alertBuilder.setTitle("Please Install Play Store");
        alertBuilder.setMessage("TextFile does not work correctly and may crash or do unexpected things if the Google Play Store is not installed. Please install the Google Play Store app and download TextFile from the Play Store.");
        alertBuilder.setNegativeButton(R.string.close, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.cancel();
            }
        });
        AlertDialog alertDialog = alertBuilder.create();
        alertDialog.show();
    }

    // Display no Google account error dialog
    public void displayNoGoogleAccountDialog(){
        AlertDialog.Builder alertBuilder = new AlertDialog.Builder(this);
        alertBuilder.setTitle("Please Add Google Account");
        alertBuilder.setMessage("TextFile does not work correctly and may crash or do unexpected things if you are not using a Google account. Please add your Google account in the settings for your device or by opening the Google Play Store app.");
        alertBuilder.setNegativeButton(R.string.close, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.cancel();
            }
        });
        AlertDialog alertDialog = alertBuilder.create();
        alertDialog.show();
    }

    // Display dialog for file limit info dialog
    public void displayDialogFileLimitInfoBar() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setMessage(R.string.dialog_file_limit_info_bar);
        builder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                launchPurchase();
            }
        });
        builder.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.cancel();
            }
        });
        AlertDialog deleteFileDialog = builder.create();
        deleteFileDialog.show();
    }

    public void fileLimitInfoBarClicked(View view) {
//        displayDialogFileLimitInfoBar();
        launchPurchase();
    }

    public void newsBarClicked(View view) {

        // Set shared preference
        SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
        SharedPreferences.Editor editor = prefs.edit();
        editor.putBoolean("settings_news_clicked", true);
        editor.commit();

        hideNewsBar();
        launchPurchase();
    }


    //---- Activities and Popups

    public void launchPurchase(){
        Intent intent = new Intent(this, PurchaseActivity.class);
        this.startActivity(intent);
        this.overridePendingTransition(0,0);
    }

    public void launchSettings(){
        Intent intent = new Intent(this, SettingsActivity.class);
        this.startActivity(intent);
        this.overridePendingTransition(0,0);
    }

    public void shareLinkMessage(){
        displayLocalMenuShareDialog();
    }

    // Launch TextFile App website
    public void launchWebsite(){
        Intent browserIntent = new Intent(Intent.ACTION_VIEW, Uri.parse("http://www.textfileapp.com"));
        startActivity(browserIntent);
    }

    //---- Choose File Service

    // Open a file or folder
    @Override
    public void onListItemClick(ListView list, View view, int position, long id){

        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        Boolean toggleShowLocalFiles = sharedPreferences.getBoolean("settings_show_local_files", true);

        if (toggleShowLocalFiles == true) {
            openFileLocal(position);
        } else {
            openFileOrFolderDropbox(position);
        }
    }

    // Add a file
    public void newFile() {

        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        Boolean toggleShowLocalFiles = sharedPreferences.getBoolean("settings_show_local_files", true);

        if (toggleShowLocalFiles == true) {
            newFileLocal();
        } else {
            newFileDropbox();
        }
    }

    // Add a folder
    public void newFolder() {
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        Boolean toggleShowLocalFiles = sharedPreferences.getBoolean("settings_show_local_files", true);
        Boolean toggleEnableProFeatures = sharedPreferences.getBoolean("settings_enable_pro_features", true);
        Boolean toggleEnablePremiumFeatures = sharedPreferences.getBoolean("settings_enable_premium_features", true);

        if (toggleShowLocalFiles == true) {
            displayLocalNewFolderDialog();
        } else if (toggleEnableProFeatures || toggleEnablePremiumFeatures) {
            displayDropboxNewFolderDialog();
        } else {
//            displayDialogNewFolderDropboxFree();
            launchPurchase();
        }
    }

    // Delete a file
    public void deleteFileOrFolder(int position){
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        Boolean toggleShowLocalFiles = sharedPreferences.getBoolean("settings_show_local_files", true);

        if (toggleShowLocalFiles == true) {
            deleteFileLocal(position);
        } else {
            deleteFileOrFolderDropbox(position);
        }
    }

    // Add a folder
    public void shareFileOrFolder(int position) {
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        Boolean toggleEnablePremiumFeatures = sharedPreferences.getBoolean("settings_enable_premium_features", true);
        Boolean toggleShowLocalFiles = sharedPreferences.getBoolean("settings_show_local_files", true);

        if (!toggleShowLocalFiles) {
            displayEnabledShareLinkDialog(position);
        } else {
            displayDisabledShareLinkDialog();
        }
    }

    //---- Local File Actions

    // View local files
    public void showFilesLocal(){

        // Get pro and premium upgrade settings
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        Boolean toggleProFeatures = sharedPreferences.getBoolean("settings_enable_pro_features", false);
        Boolean togglePremiumFeatures = sharedPreferences.getBoolean("settings_enable_premium_features", false);

        // Get directory of files
        File file = new File(getFilesDir().toString());
        File[] filesArray = file.listFiles();

        // Clear file list
        files.clear();

        // Create an array of file names without extension or gaClientId file
        int i;
        for (i = 0; i < filesArray.length; i++){
            String s1 = filesArray[i].getName().toString();
            String s2 = FilenameUtils.removeExtension(s1);
            if (s2.contains("gaClientId")){
                // Do nothing
            } else {
                files.add(s2);
            }
        }

        // Sort alphabetically
        Collections.sort(files);

        // Trim list to correct number of files depending on purchase
        if (toggleProFeatures || togglePremiumFeatures){
            textInfoBar.setVisibility(View.GONE);
        } else {
            int j = files.size();
            if (j > 5) {
                files.subList(5, j).clear();
                textInfoBar.setVisibility(View.VISIBLE);
            } else {
                textInfoBar.setVisibility(View.GONE);
            }
        }

        // Display files in ListView
        FilesLocalArrayAdapter adapter = new FilesLocalArrayAdapter(this, R.layout.list_row_files, files);
        setListAdapter(adapter);

        // Setup action bar
        getActionBar().setDisplayShowHomeEnabled(true);
        getActionBar().setHomeButtonEnabled(false);
        getActionBar().setDisplayHomeAsUpEnabled(false);
    }

    // Open local file
    public void openFileLocal(int position){

        // Get filename from ListView item
        String filename = (String) getListAdapter().getItem(position);
        String action = "open_file";

        // Launch note activity
        Intent intent = new Intent(this, EditorActivity.class);
        intent.putExtra("filename", filename);
        intent.putExtra("action", action);
        this.startActivity(intent);
        this.overridePendingTransition(0,0);
    }

    // Add a file
    public void newFileLocal() {

        // Launch a new note activity
        String action = "new_file";
        Intent intent = new Intent(this, EditorActivity.class);
        intent.putExtra("action", action);
        this.startActivity(intent);
        this.overridePendingTransition(0,0);
    }

    // Delete local file
    public void deleteFileLocal(int position) {

        // Get filename from ListView item
        String filename = (String) getListAdapter().getItem(position);

        // Show files list before deleted file or folder
        restoreState();

        // Delete the file
        deleteFile(filename + ".txt");

        // Refresh folder contents with deleted file or folder
        restoreState();
    }

    //---- Dropbox File Actions

    // View Dropbox files
    // Note - Not being used. Need to update to use custom adapter.
    public void showFilesDropbox(){
        filesNewDropbox.clear();
        foldersNewDropbox.clear();
        filesAndFoldersDropbox.clear();

        // Get path from preferences
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        String currentDbxPath = sharedPreferences.getString("setting_current_dbx_path", DbxPath.ROOT.toString());
        DbxPath tempPath = new DbxPath(currentDbxPath);
        dbxPath = tempPath;

        // Get pro and premium upgrade settings
        Boolean toggleProFeatures = sharedPreferences.getBoolean("settings_enable_pro_features", false);
        Boolean togglePremiumFeatures = sharedPreferences.getBoolean("settings_enable_premium_features", false);

        // Create folder list and file list without gaClientId file
        try {
            List<DbxFileInfo> infos = dbxFs.listFolder(dbxPath);

            for (DbxFileInfo info : infos) {
                if (info.isFolder == true){
                    foldersNewDropbox.add(info);
                } else {
                    if (info.toString().contains("gaClientId")){
                        // Do nothing
                    } else {
                        filesNewDropbox.add(info);
                    }
                }
            }
        } catch (IOException e) {
            Toast.makeText(getApplicationContext(), "Displaying Dropbox files failed", Toast.LENGTH_LONG).show();
        }

        // Sort alphabetically
        Collections.sort(foldersNewDropbox);
        Collections.sort(filesNewDropbox);

        // Add to list to display
        filesAndFoldersDropbox.addAll(filesNewDropbox);

        // Trim list to correct number of files depending on purchase
        if (toggleProFeatures || togglePremiumFeatures){
            textInfoBar.setVisibility(View.GONE);
        } else {
            int j = filesAndFoldersDropbox.size();
            if (j > 5) {
                filesAndFoldersDropbox.subList(5, j).clear();
                textInfoBar.setVisibility(View.VISIBLE);
            } else {
                textInfoBar.setVisibility(View.GONE);
            }
        }

        // Add to view through adapter
        FilesDropboxArrayAdapter adapter = new FilesDropboxArrayAdapter(this, R.layout.list_row_files, filesAndFoldersDropbox);
        setListAdapter(adapter);

        // Setup action bar
        if (dbxPath.toString().equals(DbxPath.ROOT.toString())) {
            getActionBar().setDisplayShowHomeEnabled(true);
            getActionBar().setHomeButtonEnabled(false);
            getActionBar().setDisplayHomeAsUpEnabled(false);
        } else {
            getActionBar().setDisplayShowHomeEnabled(true);
            getActionBar().setHomeButtonEnabled(true);
            getActionBar().setDisplayHomeAsUpEnabled(true);
        }
    }

    // View Dropbox files and folders
    public void showFilesFoldersDropbox(){
        filesNewDropbox.clear();
        foldersNewDropbox.clear();
        filesAndFoldersDropbox.clear();

        // Get path from preferences
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        String currentDbxPath = sharedPreferences.getString("setting_current_dbx_path", DbxPath.ROOT.toString());
        DbxPath tempPath = new DbxPath(currentDbxPath);
        dbxPath = tempPath;

        // Create folder list and file list without gaClientId file
        try {
            List<DbxFileInfo> infos = dbxFs.listFolder(dbxPath);

            for (DbxFileInfo info : infos) {
                if (info.isFolder == true){
                    foldersNewDropbox.add(info);
                } else {
                    if (info.toString().contains("gaClientId")){
                        // Do nothing
                    } else {
                        filesNewDropbox.add(info);
                    }
                }
            }
        } catch (IOException e) {
            Toast.makeText(getApplicationContext(), "Displaying Dropbox files failed", Toast.LENGTH_LONG).show();
        }

        // Sort alphabetically
        Collections.sort(foldersNewDropbox);
        Collections.sort(filesNewDropbox);

        // Add to list to display
        filesAndFoldersDropbox.addAll(foldersNewDropbox);
        filesAndFoldersDropbox.addAll(filesNewDropbox);

        // Add to view through adapter
        FilesDropboxArrayAdapter adapter = new FilesDropboxArrayAdapter(this, R.layout.list_row_files, filesAndFoldersDropbox);
        setListAdapter(adapter);

        // Setup action bar
        if (dbxPath.toString().equals(DbxPath.ROOT.toString())) {
            getActionBar().setDisplayShowHomeEnabled(true);
            getActionBar().setHomeButtonEnabled(false);
            getActionBar().setDisplayHomeAsUpEnabled(false);
        } else {
            getActionBar().setDisplayShowHomeEnabled(true);
            getActionBar().setHomeButtonEnabled(true);
            getActionBar().setDisplayHomeAsUpEnabled(true);
        }
    }

    // View parent Dropbox folder
    public void parentFolderDropbox(){
        removeFolderListener();

        if (dbxPath != DbxPath.ROOT) {
            DbxPath parent = dbxPath.getParent();
            dbxPath = parent;

            // Save path to preferences
            SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
            SharedPreferences.Editor editor = sharedPreferences.edit();
            editor.putString("setting_current_dbx_path", dbxPath.toString());
            editor.commit();
        } else {
            dbxPath = DbxPath.ROOT;

            // Save path to preferences
            SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
            SharedPreferences.Editor editor = sharedPreferences.edit();
            editor.putString("setting_current_dbx_path", DbxPath.ROOT.toString());
            editor.commit();
        }
        restoreState();
    }

    // Open a Dropbox file or folder
    public void openFileOrFolderDropbox(int position){

        removeFolderListener();

        // Get filename from ListView item
        DbxFileInfo dbxFileInfo = (DbxFileInfo) getListAdapter().getItem(position);

        if (dbxFileInfo.isFolder){
                dbxPath = dbxFileInfo.path;

                // Save path to preferences
                SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
                SharedPreferences.Editor editor = sharedPreferences.edit();
                editor.putString("setting_current_dbx_path", dbxPath.toString());
                editor.commit();

                restoreState();
        } else {
            Intent intent = new Intent(this, EditorActivity.class);
            String action = "open_file";
            String pathName = dbxFileInfo.path.toString();
            intent.putExtra("action", action);
            intent.putExtra("filename", pathName);
            this.startActivity(intent);
            this.overridePendingTransition(0,0);
        }
    }

    public void newFileDropbox(){

        removeFolderListener();

        // Launch a new note activity
        String action = "new_file";
        Intent intent = new Intent(this, EditorActivity.class);
        intent.putExtra("action", action);
        this.startActivity(intent);
        this.overridePendingTransition(0,0);
    }

    public void newFolderDropbox(String folderName){

        // Get path from preferences
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        String currentDbxPath = sharedPreferences.getString("setting_current_dbx_path", DbxPath.ROOT.toString());
        DbxPath tempPath = new DbxPath(currentDbxPath);
        dbxPath = tempPath;

        // Make full folder path
        DbxPath dbxFolder = null;
        dbxFolder = dbxPath.getChild(folderName);

        try {
            dbxFs.createFolder(dbxFolder);
        } catch (IOException e) {
            Toast.makeText(getApplicationContext(), "Creating Dropbox folder failed", Toast.LENGTH_LONG).show();
        }

        restoreState();
    }

    public void deleteFileOrFolderDropbox(int position) {

        // Get filename from ListView item
        DbxFileInfo dbxFileInfo = (DbxFileInfo) getListAdapter().getItem(position);

        // Show files list before deleted file or folder
        restoreState();

        // Delete file or folder
        if (dbxFileInfo.isFolder){
            try{
                DbxPath dbxFilePath = dbxFileInfo.path;
                dbxFs.delete(dbxFilePath);
            } catch (IOException e) {
                e.printStackTrace();
            }
        } else {
            try{
                DbxPath dbxFilePath = dbxFileInfo.path;
                dbxFs.delete(dbxFilePath);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        // Refresh folder contents with deleted file or folder
        restoreState();
    }

    public void renameFolderDropbox(int position, String folderName) {
        DbxPath oldDbxFolderPath = null;
        DbxPath newDbxFolderPath = null;

        // Get filename from ListView item
        DbxFileInfo dbxFileInfo = (DbxFileInfo) getListAdapter().getItem(position);

        // Show files list before renamed folder
        restoreState();

        // Rename file or folder
        if (dbxFileInfo.isFolder){
            try{
                oldDbxFolderPath = dbxFileInfo.path;

                // Check if root path
                if (oldDbxFolderPath.getParent() == DbxPath.ROOT){
                    newDbxFolderPath = new DbxPath(oldDbxFolderPath.getParent().toString() + folderName);
                } else {
                    newDbxFolderPath = new DbxPath( oldDbxFolderPath.getParent().toString() + "/" + folderName);
                }
                dbxFs.move(oldDbxFolderPath, newDbxFolderPath);
            } catch (IOException e) {
                e.printStackTrace();
            }
        } else {
            // Do nothing if it is file
            // Renaming file should be hidden from context menu when clicking file
        }

        // Refresh folder contents with deleted file or folder
        restoreState();
    }

    public void shareLinkCopy(int position){

        // If online
        boolean networkAvailable = isNetworkAvailable();
        if (networkAvailable) {

            // Get filename from ListView item
            DbxFileInfo dbxFileInfo = (DbxFileInfo) getListAdapter().getItem(position);
            final DbxPath dbxPath = new DbxPath(dbxFileInfo.path.toString());

            Thread fetchURL = new Thread(new Runnable() {
                @Override
                public void run() {
                    //do network action in this function
                    try {
                        URL url = dbxFs.fetchShareLink(dbxPath, false);
                        shareURL = url;
                    } catch (DbxException e) {
                        e.printStackTrace();
                    }
                }
            });

            fetchURL.start();
            try {
                fetchURL.join();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }

            // Copy to clipboard and display message
            try {
                if (dbxFileInfo.isFolder) {
                    ClipboardManager clipboard = (ClipboardManager) getSystemService(CLIPBOARD_SERVICE);
                    ClipData clip = ClipData.newPlainText("Link for notes folder " + dbxPath.getName().toString(), shareURL.toString());
                    clipboard.setPrimaryClip(clip);
                    Toast.makeText(getApplicationContext(), "Link for notes folder \"" + dbxPath.getName().toString() + "\" copied and ready to paste.\n\n" + shareURL.toString(), Toast.LENGTH_LONG).show();

                } else {
                    ClipboardManager clipboard = (ClipboardManager) getSystemService(CLIPBOARD_SERVICE);
                    ClipData clip = ClipData.newPlainText("Link for note " + FilenameUtils.removeExtension(dbxPath.getName().toString()), shareURL.toString());
                    clipboard.setPrimaryClip(clip);
                    Toast.makeText(getApplicationContext(), "Link for note \"" + FilenameUtils.removeExtension(dbxPath.getName().toString()) + "\" copied and ready to paste.\n\n" + shareURL.toString(), Toast.LENGTH_LONG).show();
                }
            } catch (NullPointerException n) {
                n.printStackTrace();
            }
        } else {
            Toast.makeText(getApplicationContext(), R.string.error_share_link, Toast.LENGTH_LONG).show();
        }
    }

    public void shareLinkEmail(int position){

        boolean networkAvailable = isNetworkAvailable();

        // If online
        if (networkAvailable) {

            // Get filename from ListView item
            DbxFileInfo dbxFileInfo = (DbxFileInfo) getListAdapter().getItem(position);
            final DbxPath dbxPath = new DbxPath(dbxFileInfo.path.toString());

            Thread fetchURL = new Thread(new Runnable() {
                @Override
                public void run() {
                    try {
                        URL url = dbxFs.fetchShareLink(dbxPath, false);
                        shareURL = url;
                    } catch (DbxException e) {
                        e.printStackTrace();
                    }
                }
            });

            fetchURL.start();
            try {
                fetchURL.join();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }

            // Create email message
            try {
                if (dbxFileInfo.isFolder) {
                    Intent emailIntent = new Intent(android.content.Intent.ACTION_SEND);

                    emailIntent.setType("plain/text");
                    emailIntent.putExtra(android.content.Intent.EXTRA_EMAIL, new String[]{""});
                    emailIntent.putExtra(android.content.Intent.EXTRA_SUBJECT, "Link for notes folder \"" + dbxPath.getName().toString() + "\" sent with TextFile");
                    emailIntent.putExtra(android.content.Intent.EXTRA_TEXT, "Link for notes folder \"" + dbxPath.getName().toString() + "\"\n• " + shareURL.toString() + "\n\n" + this.getString(R.string.text_textfile_share_email_text));

                    startActivity(emailIntent);
                } else {
                    Intent emailIntent = new Intent(android.content.Intent.ACTION_SEND);

                    emailIntent.setType("plain/text");
                    emailIntent.putExtra(android.content.Intent.EXTRA_EMAIL, new String[]{""});
                    emailIntent.putExtra(android.content.Intent.EXTRA_SUBJECT, "Link for note \"" + FilenameUtils.removeExtension(dbxPath.getName().toString()) + "\" sent with TextFile");
                    emailIntent.putExtra(android.content.Intent.EXTRA_TEXT, "Link for note \"" + FilenameUtils.removeExtension(dbxPath.getName().toString()) + "\"\n• " + shareURL.toString() + "\n\n" + this.getString(R.string.text_textfile_share_email_text));

                    startActivity(emailIntent);
                }
            } catch (NullPointerException n) {
                n.printStackTrace();
            }
        } else {
            Toast.makeText(getApplicationContext(), R.string.error_share_link, Toast.LENGTH_LONG).show();
        }
    }

    public void syncFilesDropbox(){

        List<DbxFileInfo> fileInfoList = null;

        try {
            DbxSyncStatus dbxSyncStatus = dbxFs.getSyncStatus();
            fileInfoList = dbxFs.listFolder(DbxPath.ROOT);

            // Recursively sync files in subfolders
            syncFilesInFolderAndSubfolders(fileInfoList);

            // Test method finished
            // Toast.makeText(getApplicationContext(), "Sync Done", Toast.LENGTH_LONG).show();

        } catch (DbxException e) {
            e.printStackTrace();
        } catch (IOException e) {
            Toast.makeText(getApplicationContext(), "Syncing Dropbox files failed", Toast.LENGTH_LONG).show();
        }

/*
        try {
            Thread.sleep(2000);
        } catch(InterruptedException e) {
        }
*/

        restoreState();
    }

    // Todo: Speed up syncing. Check if latest version, etc.
    // Todo: Add listener to each file upon syncing. OR each folder??? So don't need to manually sync.
    // Sync files in folder and subfolders (Recursive)
    public boolean syncFilesInFolderAndSubfolders(List<DbxFileInfo> fileInfoList){
        String fileText = null;
        DbxPath filePathToSync = null;
        DbxPath folderPathInfoToSync = null;
        DbxFile fileToSync = null;
        DbxFile fileTemp = null;
        List<DbxFileInfo> folderInfoList = null;

        try {

            for (DbxFileInfo info : fileInfoList){
                if (info.isFolder == true){

                    // Get file info
                    folderPathInfoToSync = info.path;
                    folderInfoList = dbxFs.listFolder(folderPathInfoToSync);

                    // Recursively sync files in subfolders
                    syncFilesInFolderAndSubfolders(folderInfoList);
                } else {

                    // Get file and close it later
                    filePathToSync = info.path;
                    fileToSync =  dbxFs.open(filePathToSync);
                    fileText = fileToSync.readString();
                    fileToSync.close();

                    fileTemp = dbxFs.open(filePathToSync);
                    fileTemp.writeString(fileText);
                    fileTemp.close();

//                    fileToSync.update();

                    // Check if newest version
/*                    if (fileToSync.getNewerStatus() == null){
                        // Do nothing; you have newest version of file
                        fileToSync.update();
                    } else {
                        fileToSync.update();
                    }
*/

//                    fileToSync.close();
                }
            }
        } catch (DbxException e) {
            e.printStackTrace();
        } catch (IOException e) {
            Toast.makeText(getApplicationContext(), "Syncing Dropbox files failed", Toast.LENGTH_LONG).show();
        }

        return true;
    }

    //---- Dropbox Listeners

    private void addFolderListener(){

        // Get path from preferences
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        String currentDbxPath = sharedPreferences.getString("setting_current_dbx_path", DbxPath.ROOT.toString());
        DbxPath tempPath = new DbxPath(currentDbxPath);
        dbxPath = tempPath;

        dbxFs.addPathListener(pathListener, dbxPath, DbxFileSystem.PathListener.Mode.PATH_OR_CHILD);

    }

    private void removeFolderListener(){

        // Get path from preferences
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        String currentDbxPath = sharedPreferences.getString("setting_current_dbx_path", DbxPath.ROOT.toString());
        DbxPath tempPath = new DbxPath(currentDbxPath);
        dbxPath = tempPath;

        dbxFs.removePathListener(pathListener, dbxPath, DbxFileSystem.PathListener.Mode.PATH_OR_CHILD);

    }

    //---- Settings

    public void enableProFeatures(){
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        SharedPreferences.Editor editor = sharedPreferences.edit();

        editor.putBoolean("settings_enable_pro_features", true);
        editor.commit();
//        restoreState();
    }

    public void disableProFeatures(){
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        SharedPreferences.Editor editor = sharedPreferences.edit();

        editor.putBoolean("settings_enable_pro_features", false);
        editor.commit();
//        restoreState();
    }

    public void enablePremiumFeatures(){
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        SharedPreferences.Editor editor = sharedPreferences.edit();

        editor.putBoolean("settings_enable_premium_features", true);
        editor.commit();
//        restoreState();
    }

    public void disablePremiumFeatures(){
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        SharedPreferences.Editor editor = sharedPreferences.edit();

        editor.putBoolean("settings_enable_premium_features", false);
        editor.commit();
//        restoreState();
    }

    public void disableAllFeatures(){
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        SharedPreferences.Editor editor = sharedPreferences.edit();
        editor.putBoolean("settings_enable_pro_features", false);
        editor.putBoolean("settings_enable_premium_features", false);
        editor.commit();
    }

    public void enableSaleOn(int saleSavePercent){
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        SharedPreferences.Editor editor = sharedPreferences.edit();

        editor.putBoolean("settings_enable_sale_on", true);
        editor.putInt("sale_save_percent", saleSavePercent);
        editor.commit();
//        restoreState();
    }

    public void disableSaleOn(int saleZero){
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        SharedPreferences.Editor editor = sharedPreferences.edit();

        editor.putBoolean("settings_enable_sale_on", false);
        editor.putInt("sale_save_percent", saleZero);
        editor.commit();
//        restoreState();
    }

    public void showAds(){
        AdView adViewFiles = (AdView) this.findViewById(R.id.adViewFiles);
        adViewFiles.setVisibility(View.VISIBLE);
    }

    public void hideAds(){
        AdView adViewFiles = (AdView) this.findViewById(R.id.adViewFiles);
        adViewFiles.setVisibility(View.GONE);
    }

    public void showInfoBar(){
        textInfoBar = (LinearLayout)findViewById(R.id.text_info_bar);
        textInfoBar.setVisibility(View.VISIBLE);
    }

    public void hideInfoBar(){
        textInfoBar = (LinearLayout)findViewById(R.id.text_info_bar);
        textInfoBar.setVisibility(View.GONE);
    }

    public void hideNewsBar(){
        newsInfoBar = (LinearLayout)findViewById(R.id.text_news_bar);
        newsInfoBar.setVisibility(View.GONE);
    }

    //---- Utilities

    // Check if network connection is online
    private boolean isNetworkAvailable() {
        ConnectivityManager connectivityManager
                = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetworkInfo = connectivityManager.getActiveNetworkInfo();
        return activeNetworkInfo != null && activeNetworkInfo.isConnected();
    }

    // Create a local file with name of life_thinkist
    public void createWebsiteFile() {

        try {
            FileOutputStream file = openFileOutput("textfileapp.com.txt", Context.MODE_PRIVATE);
            file.flush();
            file.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    // Check String
    public String checkString(String s) {
        String sChecked="";
        sChecked = s.replaceFirst("(.)(.)", "$2$1");
        return sChecked;
    }

    // Get account name
    public String getAccountName(){
        AccountManager am = AccountManager.get(this); // "this" references the current Context
        Account[] accounts = am.getAccountsByType("com.google");
        String accountName = "";

        if (accounts.length == 0) {
            accountName = "";
        } else {
            accountName = accounts[0].name;
        }
        return accountName;
    }

    // Check if app is installed
    private boolean isPackageInstalled(String packagename, Context context) {
        PackageManager pm = context.getPackageManager();
        try {
            pm.getPackageInfo(packagename, PackageManager.GET_ACTIVITIES);
            return true;
        } catch (PackageManager.NameNotFoundException e) {
            return false;
        }
    }

    // Count
//    private int returnNumberOfFiles

    //---- Purchases

    @Override
    public void onDestroy() {
        // Destroy the AdView.
        adViewFiles.destroy();

        super.onDestroy();
        if (mHelper != null) mHelper.dispose();
        mHelper = null;
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {

        if (mHelper == null) return;

        // Pass on the activity result to the helper for handling
        if (!mHelper.handleActivityResult(requestCode, resultCode, data)) {
            super.onActivityResult(requestCode, resultCode, data);
        }
        else {
            // Do nothing
        }
    }

    public void checkGoogleAccountPlayStoreSetPurchases(){
        boolean isPlayStoreInstalled;
        boolean isAccountBlank;
        String accountName;

        isPlayStoreInstalled = isPackageInstalled("com.android.vending", this);
        accountName = getAccountName();
        if (accountName.equals("")){
            isAccountBlank = true;
        } else {
            isAccountBlank = false;
        }

        // Play Store not installed
        if (!isPlayStoreInstalled){
            displayNoPlayStoreDialog();
            disableAllFeatures();

        // No Google account entered
        } else if (isAccountBlank) {
            displayNoGoogleAccountDialog();
            disableAllFeatures();

        // Play Store installed and Google account
        } else {

            // Setup IAB and check for purchases
            mHelper.startSetup(new IabHelper.OnIabSetupFinishedListener() {
                public void onIabSetupFinished(IabResult result) {
                    if (!result.isSuccess()) {
                        // There was a problem setting up IAB
                    }
                    // IAB is fully set up
                    if (mHelper == null) return;

                    // Inventory of products they own
//                    mHelper.queryInventoryAsync(mGotInventoryListener);

                    // Inventory of products available
                    ArrayList<String> productIDs = new ArrayList<String>();
                    productIDs.add(SKU_PRO);
                    productIDs.add(SKU_PREMIUM);
                    mHelper.queryInventoryAsync(true, productIDs, mQueryFinishedListener);
                }
            });
        }
    }

    //---- Listeners

    // Check which purchases the user has bought
    IabHelper.QueryInventoryFinishedListener mGotInventoryListener = new IabHelper.QueryInventoryFinishedListener() {
        public void onQueryInventoryFinished(IabResult result, Inventory inventory) {

            if (mHelper == null) return;

            if (result.isFailure()) {
                // handle error here
            } else {

                // Check purchases the user has
                hasPro = inventory.hasPurchase(SKU_PRO);
                hasPremium = inventory.hasPurchase(SKU_PREMIUM);

                // Update UI based on purchases
                if (hasPro) {
                    enableProFeatures();
                } else {
                    disableProFeatures();
                }

                if (hasPremium) {
                    enablePremiumFeatures();
                } else {
                    disablePremiumFeatures();
                }
            }
        }
    };

    IabHelper.QueryInventoryFinishedListener mQueryFinishedListener = new IabHelper.QueryInventoryFinishedListener() {
        public void onQueryInventoryFinished(IabResult result, Inventory inventory) {

            if (mHelper == null) return;

            if (result.isFailure()) {
                // handle error
                return;
            }

            // Check purchases the user has
            hasPro = inventory.hasPurchase(SKU_PRO);
            hasPremium = inventory.hasPurchase(SKU_PREMIUM);

            // Update UI based on purchases
            if (hasPro) {
                enableProFeatures();
            } else {
                disableProFeatures();
            }

            if (hasPremium) {
                enablePremiumFeatures();
            } else {
                disablePremiumFeatures();
            }

            // Calculate and display savings in info bar (If on sale and not upgraded)
            if (!hasPro && !hasPremium) {
                int saleSavePercent;
                double doubleTemp;

                int proPriceMicros = inventory.getSkuDetails(SKU_PRO).getPriceAmountMicros();
                int premiumPriceMicros = inventory.getSkuDetails(SKU_PREMIUM).getPriceAmountMicros();

                double proPriceMicrosDouble = (double) proPriceMicros;
                double premiumPriceMicrosDouble = (double) premiumPriceMicros;
                double proPriceDollars = proPriceMicrosDouble / 1000000;
                double premiumPriceDollars = premiumPriceMicrosDouble / 1000000;

                double costDecimal = proPriceDollars / premiumPriceDollars;
                double savingsDecimal = 1 - costDecimal;
                double savingsPercent = savingsDecimal * 100;

                if (savingsDecimal != 0) {
                    // Round down to multiple of 5
                    doubleTemp = 5 * (Math.floor(Math.abs(savingsPercent / 5)));
                    saleSavePercent = (int) doubleTemp;
                    enableSaleOn(saleSavePercent);

//                    disableSaleOn(0);

//                    textInfoBarText.setText("Save " + saleSavePercent + "%. Upgrade today.\n• Tap here for more information.");
//                    textInfoBarText.setTextColor(getResources().getColor(R.color.red));
                } else {
                    disableSaleOn(0);
//                    textInfoBarText.setText(R.string.file_limit);
//                    textInfoBarText.setTextColor(getResources().getColor(R.color.holo_light_ui_text));
                }
            }
        }
    };

    DbxFileSystem.PathListener pathListener =  new DbxFileSystem.PathListener() {
        @Override
        public void onPathChange(DbxFileSystem fs, DbxPath registeredPath, Mode registeredMode) {

            runOnUiThread(new Runnable() {
                public void run() {
                    restoreState();
                }
            });
        }
    };

}