package com.lifethinkist.textfile;

import android.accounts.Account;
import android.accounts.AccountManager;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.graphics.Paint;
import android.graphics.Typeface;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import com.android.vending.billing.util.IabHelper;
import com.android.vending.billing.util.IabResult;
import com.android.vending.billing.util.Inventory;
import com.android.vending.billing.util.Purchase;

import java.util.ArrayList;

public class PurchaseActivity extends Activity {

    IabHelper mHelper;
    private String TAG = "TextFile";

    private Button proFeaturesButton;
    private Button premiumFeaturesButton;
    private TextView textProPrice;
    private TextView textPremiumPrice;
    private TextView textPrices;

    private TextView textDealCanada;
    private TextView textDealAustralia;
    private TextView textDealNewZealand;

    private TextView textSaleSave;
    private TextView textSaleIntro;
    private TextView textSalePrice;

    private String checkedString;

    // Upgrade SKU IDs
    // Pro (source code) = Premium (upgrade)
    // Premium (source code) = Professional (upgrade)
//    static final String SKU_PRO = "subscription.pro.month";
//    static final String SKU_PREMIUM = "subscription.premium.month";
    static final String SKU_PRO = "upgrade.pro";
    static final String SKU_PREMIUM = "upgrade.premium";

    private Boolean toggleProFeaturesButton;
    private Boolean togglePremiumFeaturesButton;

    //---- Start and Stop

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        // Setup
        super.onCreate(savedInstanceState);
        setContentView(R.layout.layout_purchase);
        String base64EncodedPublicKey;

        // Setup action bar
        getActionBar().setDisplayShowHomeEnabled(true);
        getActionBar().setDisplayShowTitleEnabled(true);
        getActionBar().setHomeButtonEnabled(true);
        getActionBar().setDisplayHomeAsUpEnabled(true);

        // Setup buttons and text
        proFeaturesButton = (Button)findViewById(R.id.proFeaturesButton);
        premiumFeaturesButton = (Button)findViewById(R.id.premiumFeaturesButton);
        textProPrice = (TextView)findViewById(R.id.text_pro_price);
        textPremiumPrice = (TextView)findViewById(R.id.text_premium_price);

        textPrices = (TextView)findViewById(R.id.text_prices);
        textDealCanada = (TextView)findViewById(R.id.text_purchase_canada);
        textDealAustralia = (TextView)findViewById(R.id.text_purchase_australia);
        textDealNewZealand = (TextView)findViewById(R.id.text_purchase_new_zealand);

        textSaleSave = (TextView)findViewById(R.id.text_sale_percent);
        textSaleIntro = (TextView)findViewById(R.id.text_sale_intro);
        textSalePrice = (TextView)findViewById(R.id.text_sale_price);

        // Get preferences
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        toggleProFeaturesButton = sharedPreferences.getBoolean("settings_enable_pro_features", true);
        togglePremiumFeaturesButton = sharedPreferences.getBoolean("settings_enable_premium_features", true);

        // Setup billing
        base64EncodedPublicKey = "REMOVED";
        checkedString = checkString(base64EncodedPublicKey);
        mHelper = new IabHelper(this, checkedString);

        // Check if Play Store installed and get purchase SKUs
        checkGoogleAccountPlayStoreSetPurchases();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                // app icon in action bar clicked; go home
                launchFilesActivity();
                return true;
            case R.id.action_purchase_info:
                displayPurchaseInfoDialog();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_purchase, menu);
        return true;
    }

    //---- Activities

    // Launch main activity - rename and save done onPause
    public void launchFilesActivity (){
        Intent intent = new Intent(this, FilesActivity.class);
        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        this.startActivity(intent);
        this.overridePendingTransition(0,0);
    }

    //---- Dialogs

    // Display no Play Store error dialog
    public void displayNoPlayStoreDialog(){
        AlertDialog.Builder alertBuilder = new AlertDialog.Builder(this);
        alertBuilder.setTitle("Please Install Play Store");
        alertBuilder.setMessage("TextFile does not work correctly and may crash or do unexpected things if the Google Play Store is not installed. Please install the Google Play Store app and download TextFile from the Play Store.");
        alertBuilder.setNegativeButton(R.string.close, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.cancel();
            }
        });
        AlertDialog alertDialog = alertBuilder.create();
        alertDialog.show();
    }

    // Display no Google account error dialog
    public void displayNoGoogleAccountDialog(){
        AlertDialog.Builder alertBuilder = new AlertDialog.Builder(this);
        alertBuilder.setTitle("Please Add Google Account");
        alertBuilder.setMessage("TextFile does not work correctly and may crash or do unexpected things if you are not using a Google account. Please add your Google account in the settings for your device or by opening the Google Play Store app.");
        alertBuilder.setNegativeButton(R.string.close, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.cancel();
            }
        });
        AlertDialog alertDialog = alertBuilder.create();
        alertDialog.show();
    }

    // Display dialog with Life Thinkist description
    public void displayPurchaseInfoDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setMessage(R.string.text_purchase_info);
        builder.setPositiveButton(R.string.close, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.dismiss();
            }
        });
        AlertDialog deleteFileDialog = builder.create();
        deleteFileDialog.show();
    }
    //---- Purchases - Check info about purchases and make purchases

    IabHelper.QueryInventoryFinishedListener mQueryFinishedListener = new IabHelper.QueryInventoryFinishedListener() {
        public void onQueryInventoryFinished(IabResult result, Inventory inventory) {

            if (mHelper == null) return;

            if (result.isFailure()) {
                // handle error
                return;
            }

            // Hide purchase price if already purchased
            if (inventory.hasPurchase(SKU_PRO) || inventory.hasPurchase(SKU_PREMIUM)) {
                textProPrice.setVisibility(View.GONE);
                proFeaturesButton.setText("Unlocked!");

            // Display price details
            } else {
                // Get SKU details
                String proPrice = inventory.getSkuDetails(SKU_PRO).getPrice();
                String premiumPrice = inventory.getSkuDetails(SKU_PREMIUM).getPrice();

//            String proPriceNoDollar = proPrice.replaceAll("[$£€¥]|\\.00","");
//            String premiumPriceNoDollar = premiumPrice.replaceAll("[$£€¥]|\\.00","");
//            String proPriceCurrencyCode = inventory.getSkuDetails(SKU_PRO).getPriceCurrencyCode();
//            String premiumPriceCurrencyCode = inventory.getSkuDetails(SKU_PREMIUM).getPriceCurrencyCode();

                // Update UI
                textPrices.setText("• Premium - " + proPrice + " per month\n" +
                        "• Professional - " + premiumPrice + " per month");

//                textProPrice.setText(proPrice + " ");
//                textPremiumPrice.setText(premiumPrice + " ");

                textProPrice.setText(proPrice);
                textPremiumPrice.setText(premiumPrice);

                if (toggleProFeaturesButton) {
                    proFeaturesButton.setEnabled(false);
                    proFeaturesButton.setText("Unlocked!");
                } else {
                    proFeaturesButton.setEnabled(true);
                }

                if (togglePremiumFeaturesButton) {
                    premiumFeaturesButton.setEnabled(false);
                    premiumFeaturesButton.setText("Unlocked!");
                    proFeaturesButton.setEnabled(false);
                    proFeaturesButton.setText("Unlocked!");
                } else {
                    premiumFeaturesButton.setEnabled(true);
                }

                // Calculate sale price and savings percent
                int proPriceMicros = inventory.getSkuDetails(SKU_PRO).getPriceAmountMicros();
                int premiumPriceMicros = inventory.getSkuDetails(SKU_PREMIUM).getPriceAmountMicros();

                String savingsPercentString = null;
                boolean saleOn = false;

                double proPriceMicrosDouble = (double) proPriceMicros;
                double premiumPriceMicrosDouble = (double) premiumPriceMicros;
                double proPriceDollars = proPriceMicrosDouble / 1000000;
                double premiumPriceDollars = premiumPriceMicrosDouble / 1000000;

                double costDecimal = proPriceDollars / premiumPriceDollars;
                double savingsDecimal = 1 - costDecimal;
                double savingsPercent = savingsDecimal * 100;

                if (savingsDecimal != 0) {
                    saleOn = true;
                }

                // Display sale price if on sale
                if (saleOn){
                    int intTemp;
                    double doubleTemp;

                    doubleTemp = 5*(Math.floor(Math.abs(savingsPercent/5))); // Round down to multiple of 5
                    intTemp = (int) doubleTemp;
                    savingsPercentString = "Save " + intTemp + "%";

                    textSaleSave.setText(savingsPercentString);
                    textSalePrice.setText(proPrice);
                    textProPrice.setText(premiumPrice);
                    textProPrice.setPaintFlags(textProPrice.getPaintFlags() | Paint.STRIKE_THRU_TEXT_FLAG);

                    textSaleSave.setVisibility(View.VISIBLE);
                    textSaleIntro.setVisibility(View.VISIBLE);
                    textSalePrice.setVisibility(View.VISIBLE);
                }

                // Update UI with country specific deals
//            showDeals(proPriceCurrencyCode);
            }
        }
    };

    public void proFeaturesButtonClicked(View view) {
        String purchaseToken = getAccountName();
        if (purchaseToken == "") {
            displayNoGoogleAccountDialog();
        } else {
//            mHelper.launchPurchaseFlow(this, SKU_PRO, 10001, mPurchaseFinishedListener, purchaseToken);
            mHelper.launchPurchaseFlow(this, SKU_PRO, "inapp", 10001, mPurchaseFinishedListener, purchaseToken);
//            mHelper.launchPurchaseFlow(this, SKU_PRO, "subs", 10001, mPurchaseFinishedListener, purchaseToken);
        }
    }

    public void premiumFeaturesButtonClicked(View view) {
        String purchaseToken = getAccountName();
        if (purchaseToken == ""){
            displayNoGoogleAccountDialog();
        } else {
//            mHelper.launchPurchaseFlow(this, SKU_PREMIUM, 10001, mPurchaseFinishedListener, purchaseToken);
            mHelper.launchPurchaseFlow(this, SKU_PREMIUM, "inapp", 10001, mPurchaseFinishedListener, purchaseToken);
//            mHelper.launchPurchaseFlow(this, SKU_PREMIUM, "subs", 10001, mPurchaseFinishedListener, purchaseToken);
        }
    }

    // Check if user has made purchase when files list is created
    IabHelper.OnIabPurchaseFinishedListener mPurchaseFinishedListener = new IabHelper.OnIabPurchaseFinishedListener() {
        public void onIabPurchaseFinished(IabResult result, Purchase purchase) {

            // Get pro and premium upgrade settings
//            SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
//            SharedPreferences.Editor editor = sharedPreferences.edit();

            // give user access to premium content and update the UI
            if (result.isFailure()) {
                return;
            } else if (purchase.getSku().equals(SKU_PRO)) {
                proFeaturesButton.setEnabled(false);
                proFeaturesButton.setText("Unlocked!");
                setProEnabled();
            } else if (purchase.getSku().equals(SKU_PREMIUM)) {
                premiumFeaturesButton.setEnabled(false);
                premiumFeaturesButton.setText("Unlocked!");
                proFeaturesButton.setEnabled(false);
                proFeaturesButton.setText("Unlocked!");
                setPremiumEnabled();
            }
        }
    };

    public void checkGoogleAccountPlayStoreSetPurchases(){

        boolean isPlayStoreInstalled;
        boolean isAccountBlank;
        String accountName;

        isPlayStoreInstalled = isPackageInstalled("com.android.vending", this);
        accountName = getAccountName();
        if (accountName.equals("")){
            isAccountBlank = true;
        } else {
            isAccountBlank = false;
        }

        // Play Store not installed
        if (!isPlayStoreInstalled){
            displayNoPlayStoreDialog();

        // No Google account entered
        } else if (isAccountBlank) {
            displayNoGoogleAccountDialog();

        } else {

            // Setup IAB and get inventory of purchase SKUs
            mHelper.startSetup(new IabHelper.OnIabSetupFinishedListener() {
                public void onIabSetupFinished(IabResult result) {
                    if (!result.isSuccess()) {
                        // Oh no, there was a problem.
                    }

                    if (mHelper == null) return;

                    // Hooray, IAB is fully set up!
                    // Inventory of products available
                    ArrayList<String> productIDs = new ArrayList<String>();
                    productIDs.add(SKU_PRO);
                    productIDs.add(SKU_PREMIUM);
                    mHelper.queryInventoryAsync(true, productIDs, mQueryFinishedListener);
                }
            });
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {

        if (mHelper == null) return;

        // Pass on the activity result to the helper for handling
        if (!mHelper.handleActivityResult(requestCode, resultCode, data)) {
            super.onActivityResult(requestCode, resultCode, data);
        }
        else {
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (mHelper != null) mHelper.dispose();
        mHelper = null;
    }

    //---- Utilities

    public String checkString(String s) {
        String sChecked="";
        sChecked = s.replaceFirst("(.)(.)", "$2$1");
        return sChecked;
    }

    // Get account name. Returns blank if no account found.
    public String getAccountName(){
        AccountManager am = AccountManager.get(this); // "this" references the current Context
        Account[] accounts = am.getAccountsByType("com.google");
        String accountName = "";

        if (accounts.length == 0) {
            accountName = "";
        } else {
            accountName = accounts[0].name;
        }
        return accountName;
    }

    // Check if app is installed
    private boolean isPackageInstalled(String packagename, Context context) {
        PackageManager pm = context.getPackageManager();
        try {
            pm.getPackageInfo(packagename, PackageManager.GET_ACTIVITIES);
            return true;
        } catch (PackageManager.NameNotFoundException e) {
            return false;
        }
    }

    // Show deals for different countries
    public void showDeals(String currencyCode){

        if (currencyCode.equals("CAD")){
            textDealCanada.setVisibility(View.VISIBLE);
            textDealAustralia.setVisibility(View.GONE);
            textDealNewZealand.setVisibility(View.GONE);
        } else if (currencyCode.equals("AUD")){
            textDealCanada.setVisibility(View.VISIBLE);
            textDealAustralia.setVisibility(View.GONE);
            textDealNewZealand.setVisibility(View.GONE);
        } else if (currencyCode.equals("NZD")){
            textDealCanada.setVisibility(View.VISIBLE);
            textDealAustralia.setVisibility(View.GONE);
            textDealNewZealand.setVisibility(View.GONE);
        } else {
            // Do nothing
        }
    }

    public void setProEnabled() {
        // Get pro and premium upgrade settings
            SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
            SharedPreferences.Editor editor = sharedPreferences.edit();
            editor.putBoolean("settings_enable_pro_features", true);
            editor.commit();
    }

    public void setPremiumEnabled() {
        // Get pro and premium upgrade settings
            SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
            SharedPreferences.Editor editor = sharedPreferences.edit();
            editor.putBoolean("settings_enable_premium_features", true);
            editor.commit();
    }
}
