package com.lifethinkist.textfile;

import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.text.method.LinkMovementMethod;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.Toast;

import com.dropbox.sync.android.DbxAccountManager;
import com.dropbox.sync.android.DbxFile;
import com.dropbox.sync.android.DbxFileSystem;
import com.dropbox.sync.android.DbxPath;

import org.apache.commons.io.FileUtils;

import java.io.File;
import java.io.IOException;

public class SettingsActivity extends Activity {

    private DbxAccountManager mDbxAcctMgr;
    private static final String APP_KEY = "REMOVED";
    private static final String APP_SECRET = "REMOVED";
    private static final int REQUEST_LINK_TO_DBX = 0;

    private DbxFileSystem dbxFs;

    final static String TAG = "TextFile";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        // Setup
        super.onCreate(savedInstanceState);

        // Setup dropbox
        mDbxAcctMgr = DbxAccountManager.getInstance(getApplicationContext(), APP_KEY, APP_SECRET);

        // Setup views
        setContentView(R.layout.layout_settings);

        // Setup links
        /*TextView textViewTwitter = (TextView) findViewById(R.id.company_twitter);
        TextView textViewCompany = (TextView) findViewById(R.id.company_website_link);
        TextView textViewApps = (TextView) findViewById(R.id.company_apps_link);
        textViewTwitter.setMovementMethod(LinkMovementMethod.getInstance());
        textViewCompany.setMovementMethod(LinkMovementMethod.getInstance());
        textViewApps.setMovementMethod(LinkMovementMethod.getInstance());*/

        // Setup action bar
        getActionBar().setDisplayShowHomeEnabled(true);
        getActionBar().setDisplayShowTitleEnabled(true);
        getActionBar().setHomeButtonEnabled(true);
        getActionBar().setDisplayHomeAsUpEnabled(true);

        // Restore state
        restoreState();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_settings, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                // app icon in action bar clicked; go home
                launchFilesActivity();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    // Launch main activity - rename and save done onPause
    public void launchFilesActivity (){
        Intent intent = new Intent(this, FilesActivity.class);
        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        this.startActivity(intent);
        this.overridePendingTransition(0,0);
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == REQUEST_LINK_TO_DBX) {
            if (resultCode == Activity.RESULT_OK) {
                // ... Start using Dropbox files.

                SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
                SharedPreferences.Editor editor = sharedPreferences.edit();
                editor.putBoolean("settings_dropbox_link", true);
                editor.putBoolean("settings_show_local_files", false);
                editor.commit();
                restoreState();

                copyLocalToDropbox();
            } else {
                // ... Link failed or was cancelled by the user.

                restoreState();
                Toast.makeText(getApplicationContext(), "Link to dropbox failed. Make sure you are online " +
                        "or try again later.", Toast.LENGTH_LONG).show();
            }
        } else {
            super.onActivityResult(requestCode, resultCode, data);
        }
    }

    public void restoreState(){
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);

        Boolean toggleDropboxLink = sharedPreferences.getBoolean("settings_dropbox_link", false);
        Boolean toggleShowLocalFiles = sharedPreferences.getBoolean("settings_show_local_files", true);

        Switch switchDropboxLink = (Switch)findViewById(R.id.switch_dropbox_link);
        Switch switchShowsLocalFiles = (Switch)findViewById(R.id.switch_show_local_files);

        if (toggleDropboxLink == true) {
            switchDropboxLink.setChecked(true);
        } else {
            switchDropboxLink.setChecked(false);
        }

        if (toggleShowLocalFiles == true) {
            switchShowsLocalFiles.setChecked(true);
        } else {
            switchShowsLocalFiles.setChecked(false);
        }
    }

    public void onClickLinkDropbox(View view){
        boolean on = ((Switch) view).isChecked();

        if (on) {
            mDbxAcctMgr.startLink((Activity) this, REQUEST_LINK_TO_DBX);

        } else {
            mDbxAcctMgr.unlink();

            SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
            SharedPreferences.Editor editor = sharedPreferences.edit();
            editor.putBoolean("settings_dropbox_link", false);
            editor.putBoolean("settings_show_local_files", true);
            editor.putString("setting_current_dbx_path", "/");
            editor.commit();
            restoreState();
        }
    }

    public void onClickShowLocalFiles(View view){
        boolean on = ((Switch) view).isChecked();

        if (on) {
            SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
            SharedPreferences.Editor editor = sharedPreferences.edit();
            editor.putBoolean("settings_show_local_files", true);
            editor.commit();
        } else {
            SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
            SharedPreferences.Editor editor = sharedPreferences.edit();
            editor.putBoolean("settings_show_local_files", false);
            editor.commit();
        }
    }

    // Copy all local files to dropbox files
    public void copyLocalToDropbox(){
        int i;
        DbxFile dropboxFile = null;

        // Setup dropbox
        mDbxAcctMgr = DbxAccountManager.getInstance(getApplicationContext(), APP_KEY, APP_SECRET);
        if (mDbxAcctMgr.hasLinkedAccount()){
            try {
                dbxFs = DbxFileSystem.forAccount(mDbxAcctMgr.getLinkedAccount());
            } catch (IOException e) {
                Toast.makeText(getApplicationContext(), "Copying local files to Dropbox failed", Toast.LENGTH_LONG).show();
            }
        }

        // Create array of local files
        File file = new File(getFilesDir().toString());
        File[] filesArray = file.listFiles();

        // Copy contents of local file to dropbox file
        for (i = 0; i < filesArray.length; i++){
            String filename = filesArray[i].getName().toString();

            if (filename.contains("gaClientId")){
                // Do nothing
            } else {
                try {
                    DbxPath path = new DbxPath("/"+filename);
                    dropboxFile = dbxFs.create(path);
                    String fileData = FileUtils.readFileToString(filesArray[i]);
                    dropboxFile.writeString(fileData);
                    dropboxFile.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
